/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.system;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import jreqtools.*;

public final class ApplicationPreferences {
	private static final boolean DEBUG = false;

	// Variables
	Properties properties 	= new Properties();
	Hashtable id2object	= new Hashtable();
	
	// Property: applicationName
	String applicationName;
	
	public ApplicationPreferences(String applicationName)  {
		this.applicationName = applicationName;
		load();
	}

	public void add(String id, Object object)  {
		if (object == null || id == null) return;
		
		// In die Tabelle eintragen
		id2object.put(id, object);
		
		// Nachschauen, ob Daten vorliegen
		if (object instanceof JFrame)
			{
			Rectangle bounds = getBounds(id);
			if (bounds != null) ((JFrame)object).setBounds(bounds);
			}
		else if (object instanceof JTextComponent)
			{
			String text = getText(id);
			if (text != null) ((JTextComponent)object).setText(text);
			}
		else if (object instanceof JFileChooser)
			{
			Rectangle bounds = getBounds(id);
			if (bounds != null) ((JFileChooser)object).setBounds(bounds);
			File file = getFile(id);
			if (file != null) ((JFileChooser)object).setSelectedFile(file);
			}
		else System.err.println("ApplicationPreferences.add(): not supported: " + object.getClass().getName());
	}

	/**
	* Returns the value of the applicationName property.
	*/

	public String getApplicationName()  {
		return applicationName;
	}

	public Rectangle getBounds(String id)  {
		String s = properties.getProperty(id + ".bounds");
		if (s == null) return null;
		
		StringTokenizer tok = new StringTokenizer(s, ",");
		
		int x 	= Integer.parseInt(tok.nextToken());
		int y 	= Integer.parseInt(tok.nextToken());
		int w 	= Integer.parseInt(tok.nextToken());
		int h	= Integer.parseInt(tok.nextToken());
		
		String version = System.getProperty("java.fullversion");
		if (version != null && version.startsWith("J2RE 1.3.0 IBM build cx130-20010207"))
			{
			// Fehlerkorrektur bei der Fensterpositionierung
			x -= 5;
			y -= 44;
			}
		else if (version != null && version.startsWith("J2RE 1.3.0 IBM build cx130-20010626"))
			{
			// Fehlerkorrektur bei der Fensterpositionierung
			x -= 5;
			y -= 44;
			}
		
		return new Rectangle(x,y,w,h);
	}

	public File getFile(String id)  {
		String s = properties.getProperty(id + ".file");
		return s != null ? new File(s) : null;
	}

	public Font getFont(String id)  {
		String s = properties.getProperty(id + ".font");
		if (s == null) return null;
		
		StringTokenizer tok = new StringTokenizer(s, ",");
		
		String name = tok.nextToken();
		int size	= Integer.parseInt(tok.nextToken());
		
		return new Font(name, Font.PLAIN, size);
	}

	public Rectangle getRectangle(String id)  {
		return getBounds(id);
	}

	public String getText(String id)  {
		return properties.getProperty(id + ".text");
	}

	private void load()  {
		try
			{
			String filename = System.getProperty("user.home") + File.separator + "." + applicationName + ".prefs";
			if (DEBUG) Logger.log("reading prefs from " + filename);
			InputStream in = new FileInputStream(filename);
			properties.load(in);
			in.close();
			}
		catch (FileNotFoundException x1) { }
		catch (IOException x2) { System.err.println(x2); }
	}

	public void save()  {
		// Daten in die Properties bernehmen
		storeData();
		
		// Abspeichern
		try
			{
			String filename = System.getProperty("user.home") + File.separator + "." + applicationName + ".prefs";
			if (DEBUG) Logger.log("saving prefs to " + filename);
			OutputStream out = new FileOutputStream(filename);
			properties.store(out,"");
			out.close();
			}
		catch (IOException x2) { System.err.println(x2); }
	}

	public void setBounds(String id, Rectangle bounds)  {
		properties.put(id + ".bounds", bounds.x + "," + bounds.y + "," + bounds.width + "," + bounds.height);
	}

	public void setFile(String id, File file)  {
		if (file == null) properties.remove(id + ".file");
		else properties.put(id + ".file", file.getAbsolutePath());
	}

	public void setFont(String id, Font font)  {
		properties.put(id + ".font", font.getName() + "," + font.getSize());
	}

	public void setRectangle(String id, Rectangle bounds)  {
		setBounds(id, bounds);
	}

	public void setText(String id, String text)  {
		if (text == null) properties.remove(id + ".text");
		else properties.put(id + ".text", text);
	}

	private void storeData()  {
		for (Enumeration enum = id2object.keys(); enum.hasMoreElements(); )
			{
			String id = (String) enum.nextElement();
			storeData(id);
			}
	}

	private void storeData(String id)  {
		Object object = id2object.get(id);
		
		if (object instanceof JFrame)
			{
			Rectangle bounds = ((JFrame)object).getBounds();
			setBounds(id, bounds);
			}
		else if (object instanceof JTextComponent)
			{
			String text = ((JTextComponent)object).getText();
			setText(id, text);
			}
		else if (object instanceof JFileChooser)
			{
			File file = ((JFileChooser)object).getSelectedFile();
			setFile(id, file);
			}
		else System.err.println("ApplicationPreferences.storeData(): not supported: " + object.getClass().getName());
	}

}
